#ifndef __LISTNODE_H__
#define __LISTNODE_H__

#define		ASSERT(x)	assert(x)		// Define it as you want

/*---------------------------------------------------------
**	clase TListNode 
**
**		Class List/Node of list.
**
**		Mostly like a simple double linked list, but 
**		instead of a reference to the previous element,
**		it has a reference to the pointer that point the
**		current element. 
**
**		Each node represent the list that starts in that 
**		node and continues with the next nodes.
**	
**		An empty list is represented by a null pointer.
**		Example:
**					TListNode  *ppList=NULL;
**				Where ppList is an empty list.
**
**		Ideal for items that are moving frequently from
**		a list to other. 
**			By example movile objects that changue from the
**			the list from an space partition node to other 
**			list of a diferent node.
**
**-------------------------------------------------------*/
class TListNode
{
public:
	TListNode	**ppPrev;	// Pointer to the pointer that point us.
	TListNode	 *pNext;	// Pointer to the next node in the list.

public:
	inline	TListNode * Next()  { return pNext; } 

public:
	/*---------------------------------------------------------
	**	AssertValid
	**					Very strong invariant.
	**-------------------------------------------------------*/	
	inline	void	AssertValid()
	{
		#ifdef _DEBUG
			if( pNext != NULL )
			{
				ASSERT( (pNext->ppPrev  == &pNext) );
				ASSERT( pNext != this );
				pNext->AssertValid(); 
			}
			if( ppPrev != NULL )
			{
				ASSERT( ppPrev != &pNext );
				ASSERT( *ppPrev == this  );
			}
		#endif
	}

	/*---------------------------------------------------------
	**	AddTo
	**			Move the list headed by the current node	
	**				to the end of the given list.
	**
	**			Usefull to concatenate list.
	**
	**-------------------------------------------------------*/
	inline	void	AddTo( TListNode **new_ppPrev )
	{
		#ifdef _DEBUG
			AssertValid();
			if( *new_ppPrev != NULL ) (*new_ppPrev)->AssertValid();
		#endif

		//-------------------------------------------------
		//	1 Locate the last element of the given list.
		//-------------------------------------------------
		TListNode **cur_ppPrev = new_ppPrev;
		while( *cur_ppPrev != NULL )
		{
			cur_ppPrev = &((*cur_ppPrev)->pNext);
		}

		//-------------------------------------------------
		//	2 Move the current node to the end of the 
		//		given list.
		//-------------------------------------------------

			//-----------------------------------------------------
			//	2.1 Unlink from previous nodes.
			//-----------------------------------------------------
			if( ppPrev != NULL )
			{
				*ppPrev = NULL;
			}

			//-----------------------------------------------------
			//	2.2 Link after the last element of the given list.
			//-----------------------------------------------------
			ppPrev  = cur_ppPrev;
			*ppPrev = this;

		AssertValid();
	}

	/*---------------------------------------------------------
	**	InsertAt
	**
	**			Inserta el elemento en el puntero pasado.
	**			Enlaza la referencia dada, a la actual y
	**			la actual a la anterior, que tena.
	**
	**-------------------------------------------------------*/
	inline	void	InsertAt( TListNode **new_ppPrev )
	{
		#ifdef _DEBUG
			ASSERT( new_ppPrev != NULL );
			AssertValid();
			if( *new_ppPrev != NULL ) (*new_ppPrev)->AssertValid();
		#endif	

		//-------------------------------------------------
		//	1 Unlink from previous list.
		//-------------------------------------------------
		Delete();

		//-------------------------------------------------
		//	2 Insert in the given list.
		//-------------------------------------------------
		ASSERT( new_ppPrev!=NULL );
		ASSERT( *new_ppPrev==NULL || (*new_ppPrev)->ppPrev == new_ppPrev );

		// Links current with next node and viceversa.
		pNext		= *new_ppPrev;	// Links current with next.
		if( *new_ppPrev != NULL )	// Links next with current.
			(*new_ppPrev)->ppPrev = &(this->pNext);

		// Links previous with current node and viceversa.
		ppPrev		= new_ppPrev;	// Links current with previous.
		*ppPrev		= this;			// Links previos with current.

		AssertValid();
	}

	/*---------------------------------------------------------
	**	Delete()
	**			Unlink current element, and left the list valid.
	**-------------------------------------------------------*/
	inline	void	Delete()
	{	
		AssertValid();

		if( pNext != NULL )
		{
			pNext->ppPrev = ppPrev;
		}

		if( ppPrev!= NULL )
		{
			*ppPrev = pNext;
			ppPrev  = NULL;
		}
		pNext	= NULL;

		AssertValid();
	}

	/*---------------------------------------------------------
	** Constructor / Destructor
	**-------------------------------------------------------*/
	inline	TListNode() 
	{ 
		ppPrev = NULL;
		pNext  = NULL;
	}
};

/*---------------------------------------------------------
**
**	template class for TListNode.
**
**		Represents a ListNode with elements of type T.
**
**		Adds automatic and easy access typecasting.
**		So you can use TListN<T> as an object of type T.
**
**-------------------------------------------------------*/
template <class T> class TListN : private TListNode
{
private:
	T			   data;
public:
	//
	//	Automatic type casting for transparent access to data.
	//
	inline	T&			   operator()()  { return data;  }		// Easy access cast.
	inline	T&			   Get()         { return data;  }		// Other easy access cast.
	inline	operator	   T&()	         { return data;  }		// Automatic cast.
	inline	TListN<T>    * Next()        { return (TListN<T>*) pNext; } 

	//
	//	Ensures we only add nodes of type T to list of type T.
	//
	inline	void	AddTo(	TListN<T> **new_ppPrev )	{ TListNode::AddTo( (TListNode **)new_ppPrev );		}
	inline	void	InsertAt( TListN<T> **new_ppPrev )	{ TListNode::InsertAt( (TListNode **)new_ppPrev );  }
	inline	void	Delete()							{ TListNode::Delete();								}

	//
	//	Default constructors.
	//
	inline	TListN<T>()				   : data()		{ }		// Default empty constructor.
	inline	TListN<T>( const T &init ) : data(init) { }		// Default copy constructor.
};

template <class T> inline	void	 Advance( TListN<T> *(&pNode) )  { pNode = (TListN<T>*)pNode->Next(); }

#endif // __LISTNODE_H__
